% CLASSIFIER - adjust logistic regression function 
%
% Params:
%    X - feature vectors
%    y - real value vector (1 or 0 at each cell)
%
% Return:
%    theta - theta for prediction (function predict)
%------------------------------------------------------------------------
function theta = classifier(X, y)
  initial_theta = zeros(size(X, 2), 1);
  lambda = 1;

  options = optimset('GradObj', 'on', 'MaxIter', 400);
  [theta, J, exit_flag] = ...
	fminunc(@(t)(costFunctionReg(t, X, y, lambda)), initial_theta, options);
%------------------------------------------------------------------------
function [J, grad] = costFunctionReg(theta, X, y, lambda)
  m = length(y); 
  J = 0;
  grad = zeros(size(theta));

  g = sigmoid(theta' * X');
  s = -y' * (log(g))' - (1 - y)' * (log(1 - g))';
  ss = sum(theta .^ 2) - theta(1) ^ 2;
  J = (1/m) * s + (lambda/(2 * m)) * ss;

  s2 = (g - y') * X;
  grad = s2;
  n = length(theta);
  for iter = 2 : n
    grad(iter) += lambda * theta(iter);
  end
  grad = (1/m) * grad;
end
%------------------------------------------------------------------------
function g = sigmoid(z)
  g = zeros(size(z));
  g = 1 ./ (1 + (e .^ (-z)));
end
%------------------------------------------------------------------------
end
