% EDGELINK - scan image and return edge list
%
% Params:
%    im - binary image
%
% Return:
%    edgelist - list of edges
%    edgeim - labled image
%------------------------------------------------------------------------
function [edgelist, edgeim] = edgelink(im)
  global EDGEIM;
      
  global ROWS;
  global COLS;
  global RJ;
  global CJ;
    
  EDGEIM = im ~= 0;                     
  EDGEIM = bwmorph(EDGEIM,'clean');     
  EDGEIM = bwmorph(EDGEIM,'skel',Inf);  % 'thin'

  EDGEIM = double(EDGEIM); 
  [ROWS, COLS] = size(EDGEIM);
  edgeNo = 0;
        
    
  for r = 1:ROWS
    for c = 1:COLS
      if EDGEIM(r,c) == 1
        edgepoints = trackedge(r,c, edgeNo);
        if ~isempty(edgepoints)
          edgeNo = edgeNo + 1;                    
          edgelist{edgeNo} = edgepoints;
        end
      end
    end
  end
  edgeim = -EDGEIM;
end
%------------------------------------------------------------------------ 
function edgepoints = trackedge(rstart, cstart, edgeNo)
    
    global EDGEIM;
    global RJ;
    global CJ;
    global noPoint;
    global thereIsAPoint;
    global lastPoint;    

    noPoint = 0;
    thereIsAPoint = 1;
    lastPoint = 2;
    
    edgepoints = [rstart cstart];     
    EDGEIM(rstart,cstart) = -edgeNo; 
                                     
    
    [status, r, c] = nextpoint(rstart,cstart, edgeNo); 
                                                       
    while status ~= noPoint
        edgepoints = [edgepoints             
                       r    c   ];
        EDGEIM(r,c) = -edgeNo;               

        if status == lastPoint              
            status = noPoint;               
        else
            [status, r, c] = nextpoint(r,c, edgeNo);
        end
    end
    
    if ~isjunction(rstart,cstart)        
      edgepoints = flipud(edgepoints);  
        
      [status, r, c] = nextpoint(rstart,cstart, edgeNo); 
        
      while status ~= noPoint
        edgepoints = [edgepoints
                          r    c   ];
        EDGEIM(r,c) = -edgeNo;
        if status == lastPoint
          status = noPoint;
        else
          [status, r, c] = nextpoint(r,c, edgeNo);
        end
      end
    end
    
    if length(edgepoints) >= 4
        if abs(edgepoints(1,1) - edgepoints(end,1)) <= 1  &&  ...
           abs(edgepoints(1,2) - edgepoints(end,2)) <= 1 
            edgepoints = [edgepoints
                          edgepoints(1,:)];
        end
    end 
end
%------------------------------------------------------------------------
function [status, nextr, nextc] = nextpoint(rp,cp, edgeNo)

    global EDGEIM;
    global ROWS;
    global COLS;
    global RJ;
    global CJ;
    global noPoint;
    global thereIsAPoint;
    global lastPoint;        
    
    roff = [-1  0  1  0 -1 -1  1  1];
    coff = [ 0  1  0 -1 -1  1  1 -1];

    r = rp+roff;
    c = cp+coff;
    
    ind = find((r>=1 & r<=ROWS) & (c>=1 & c<=COLS));
    
    for i = ind 
        if (any(c(i) == CJ(RJ==r(i)))) && (EDGEIM(r(i),c(i)) ~= -edgeNo) 
            nextr = r(i);
            nextc = c(i);
            status = lastPoint;
            return;         
        end
    end
    checkFlag = 0;
    for i = ind
        if EDGEIM(r(i),c(i)) == 1
            n = neighbours(r(i),c(i));
            if sum(n==-edgeNo) < 2
                nextr = r(i);
                nextc = c(i);
                status = thereIsAPoint;
                return;            
            
            else                
                checkFlag = 1;
                rememberr = r(i);
                rememberc = c(i);               
            end
            
        end
    end
    
    if checkFlag      
        nextr = rememberr;
        nextc = rememberc;
        status = thereIsAPoint;       
        return;  
    end
        
    nextr = 0;   
    nextc = 0;
    status = noPoint;
end
%------------------------------------------------------------------------
function b = isjunction(r,c)
    global RJ;
    global CJ;
    
    b = any(c == CJ(RJ==r));
end
%------------------------------------------------------------------------
function n = neighbours(rp, cp)
    
    global EDGEIM;
    global ROWS;
    global COLS;

    roff = [-1  0  1  1  1  0 -1 -1];
    coff = [-1 -1 -1  0  1  1  1  0];
    
    r = rp+roff;
    c = cp+coff;
    
    ind = find((r>=1 & r<=ROWS) & (c>=1 & c<=COLS));    

    n = zeros(1,8);
    for i = ind
        n(i) = EDGEIM(r(i),c(i));    
    end
end
%------------------------------------------------------------------------

