% FINGERDISTANCES - return distance to nearest "probably" finger and distance to hull
%
% Params:
%    h - list of "probably" fingers (from function hough)
%    x, y - hull params (from function createvect)
%
% Return:
%    d - distance vector
%------------------------------------------------------------------------
function d = fingerdistances(h, x, y)
  pd = pointdist(h);
  hd = hulldist(h, x, y);
  d = [pd, hd];
%------------------------------------------------------------------------
function pd = pointdist(h)
  n = length(h);
  A = zeros(n, n);
  for i = 1:n-1
    p1 = h{i};
    for j = i+1:n
      p2 = h{j};
      d = sqrt(sum((p1 - p2) .^ 2));
      A(i, j) = d;
      A(j, i) = d;
    end
  end
  A += eye(n, n) * 10000000;
  pd = min(A)';
end
%------------------------------------------------------------------------
function hd = hulldist(h, x, y)
  n = length(h);
  hd = zeros(n, 1);
  ind = zeros(n, 1);
  for i = 1:n
    p = h{i};
    hd(i) = hulldistance(p, x, y);
    %[hd(i), tmp] = hulldistpoint(p, hull);
    %ind(i) = tmp;
  end
end
%------------------------------------------------------------------------
function [hd, ind] = hulldistpoint(p, hull)
  n = length(hull);
  l = hull{1};
  hd = (l(1) * p(1) + l(2) * p(2) + l(3))/(sqrt(sum(l .^ 2)));
  ind = 1;
  for i = 2:n
    l = hull{i};
    tmp = (l(1) * p(1) + l(2) * p(2) + l(3))/(sqrt(sum(l .^ 2)));
    if l < hd
      hd = l;
      ind = i;
    end
  end
end
%------------------------------------------------------------------------
function r = hulldistance(p, x, y)
n = length(x);
r = linedistance(p(1), p(2), x(1), y(1), x(2), y(2));
for i = 3:n
  tmp = linedistance(p(1), p(2), x(i - 1), y(i - 1), x(i), y(i));
  if tmp < r
    r = tmp;
  end
end
r = sqrt(r);
end
%------------------------------------------------------------------------
function r = linedistance(x, y, x0, y0, x1, y1)
dx = x1 - x0;
dy = y1 - y0;
px = x - x0;
py = y - y0;
t = dx * px + dy * py;

if t <= 0
  r = px * px + py * py;
else 
  s = dx * dx + dy* dy;
  if t >= s
    px = x - x1;
    py = y - y1;
    r = px * px + py * py;
  else
    r = px * px + py * py - (t/s) * t;
  end
end
end
%------------------------------------------------------------------------
end
