function [h, A, v] = match_hough(ccanny)
  [start, num, step] = get_finger_size(ccanny);
  [f, delta] = buildFilters(start, step, num, sum(sum(ccanny)));
  tic
  A = gethough(ccanny, f);
  toc
   h = getlocalmin(A, delta, step);
  %h = getlocalminsep(A, delta, step);
  h = deleteduplicates(h);
  v = visualizing(ccanny, h);
%------------------------------------------------------------------------------------------------------------------------------
function [f, delta] = buildFilters(start, step, num, w)
  r = start;
  for i = 1:num
   [f0, d] = buildCircle(r, w);
    f{i} = f0;
    delta{i} = d;
    r = r + step;
  endfor
end
%------------------------------------------------------------------------------------------------------------------------------
function [start, num, step] = get_finger_size(ccanny)
  maxs = min(size(ccanny) ./ 8);
  start =  int32(max(size(ccanny) ./ 100));
  step = 5;
  num = int32((maxs - start) / step);
end
%------------------------------------------------------------------------------------------------------------------------------
function [win_add, s_x, e_x, s_y, e_y] = calc_point_weight(ccanny, x, y, f)
  [n0, n1] = size(ccanny);
  n_orig = size(f);
  n = (n_orig .- 1) ./ 2;
  
  dx_n = x - n(1);
  dx_p = x + n(1);
  dy_n = y - n(2);
  dy_p = y + n(2);
  s_x = max(dx_n, 1);
  e_x = min(dx_p, n0);
  s_y = max(dy_n, 1);
  e_y = min(dy_p, n1);
  
  win = ccanny(s_x : e_x, s_y : e_y);
  win_f = f((1 + (s_x - dx_n)) : (n_orig(1) - (dx_p - e_x)), (1 + (s_y - dy_n)) : (n_orig(2) - (dy_p - e_y)));
  %win_add = win .* win_f;
  win_add = win_f;
end
%------------------------------------------------------------------------------------------------------------------------------
function A = gethough(ccanny, f)
  [n, m] = size(ccanny);
  n1 = n;
  m1 = m;
  l = length(f);
  A = zeros(n1, m1, l);
  for i = 1:n
    for j = 1:m
      if ccanny(i, j) == 1
        for k = 1:l
          A = updateMatrix(A, i, j, k, ccanny, f{k});
        end
      end
    end
  end
end
%------------------------------------------------------------------------------------------------------------------------------
function A = updateMatrix(A,  x, y, l, ccanny, f)
   [win_add, s_x, e_x, s_y, e_y] = calc_point_weight(ccanny, x, y, f);
  A(s_x : e_x, s_y : e_y, l) = A(s_x : e_x, s_y : e_y, l) + win_add;
end
%------------------------------------------------------------------------------------------------------------------------------
function h = getlocalmin(A, delta, step)
[n, m, l] = size(A);
i = 1;

r = 1;
while r < l
while 1
  %[val, x, y, r] = getmax(A);
  [val, x, y] = getmax2(A, r);
  if val > delta{r}
    %printf("%d: (%d %d) - %d\n", val, x, y, r);
    h{i} = [x, y, (r + 1) * step];
    A = makeNull(A, x, y, r, n, m, 10);
    i++;
  else
    break;
  end
end
r++;
end

end
%----------------------------------------------------------------------------
function h = getlocalminsep(A, delta, step)
[n, m, l] = size(A);
i = 1;

for r = 1:l
  %r = l;
  while 1
    %[val, x, y, r] = getmax(A);
    [val, x, y] = getmax2(A, r);
    if val > delta(r)
      %printf("%d: (%d %d) - %d\n", val, x, y, r);
      h{i} = [x, y, (r + 1) * step];
      A = makeNullSep(A, x, y, r, n, m, 10);
      i++;
    else
      break;
    end
  end
  end
end
%--------------------------------------------------------------------------
function A = makeNull(A, x, y, r, n, m, t)
  A(x, y, r) = 0;
  x1 = max(1, x-t);
  y1 = max(1, y-t);
  x2 = min(n, x+t);
  y2 = min(m, y+t);
  A(x1:x2, y1:y2, 1:r) = 0;
end
%----------------------------------------------------------------------------
function A = makeNullSep(A, x, y, r, n, m, t)
  A(x, y, r) = 0;
  x1 = max(1, x-t);
  y1 = max(1, y-t);
  x2 = min(n, x+t);
  y2 = min(m, y+t);
  A(x1:x2, y1:y2, r) = 0;
end
%-------------------------------------------------------------------
function [val, x, y, r] = getmax2(A, r)
  [mm, ind] = max(A(:, :, r));
  [mm2, ind2] = max(mm);
  y = ind2;
  x = ind(y);
  val = A(x, y, r);
end
%-------------------------------------------------------------------
function [val, x, y, r] = getmax3(A)
  [mm, ind] = max(A);
  [mm2, ind2] = max(mm);
  [mm3, ind3] = max(mm2);
  r = ind3;
  y = ind2(1, 1, r);
  x = ind(1, y, r);
  val = A(x, y, r);
end
%------------------------------------------------------------------------------------------------------------------------------
function ret = deleteduplicates(h)
  delta = 60;
  l = length(h);
  if l == 0
    return;
  end
  ind = 2;
  ret{1} = h{1};
  for i = 2:l
    t = true;
    ll = length(ret);
    tmp2 = h{i};
    for j = 1:ll
      tmp1 = ret{j};
      x = tmp1(1) - tmp2(1);
      y = tmp1(2) - tmp2(2);
      d = sqrt(x ^2 + y ^2);
      if d < tmp1(3) || d < tmp2(3)
        t = false;
        break;
      end
    end
    if t
      ret{ind} = tmp2;
      ind++;
    end
  end
end
%------------------------------------------------------------------------------------------------------------------------------
end